@@ -1,6 +1,7 @@
 ;;; em-dirs.el --- directory navigation commands
 
-;; Copyright (C) 1999, 2000 Free Software Foundation
+;; Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004,
+;;   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 
 ;; Author: John Wiegley <johnw@gnu.org>
 
@@ -8,7 +9,7 @@
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
+;; the Free Software Foundation; either version 3, or (at your option)
 ;; any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
@@ -18,12 +19,13 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 (provide 'em-dirs)
 
 (eval-when-compile (require 'esh-maint))
+(require 'eshell)
 
 (defgroup eshell-dirs nil
   "Directory navigation involves changing directories, examining the
@@ -172,7 +174,7 @@ thing again."
   :type 'boolean
   :group 'eshell-dirs)
 
-;;; Internal  Variables:
+;;; Internal Variables:
 
 (defvar eshell-dirstack nil
   "List of directories saved by pushd in the Eshell buffer.
@@ -211,7 +213,6 @@ Thus, this does not include the current directory.")
 		      'eshell-dirs-substitute-cd)
 		eshell-interpreter-alist)))
 
-  (make-local-hook 'eshell-parse-argument-hook)
   (add-hook 'eshell-parse-argument-hook
 	    'eshell-parse-user-reference nil t)
   (if (eshell-under-windows-p)
@@ -219,7 +220,6 @@ Thus, this does not include the current directory.")
 		'eshell-parse-drive-letter nil t))
 
   (when (eshell-using-module 'eshell-cmpl)
-    (make-local-hook 'pcomplete-try-first-hook)
     (add-hook 'pcomplete-try-first-hook
 	      'eshell-complete-user-reference nil t))
 
@@ -231,7 +231,6 @@ Thus, this does not include the current directory.")
   (unless eshell-last-dir-ring
     (setq eshell-last-dir-ring (make-ring eshell-last-dir-ring-size)))
 
-  (make-local-hook 'eshell-exit-hook)
   (add-hook 'eshell-exit-hook 'eshell-write-last-dir-ring nil t)
 
   (add-hook 'kill-emacs-hook 'eshell-save-some-last-dir))
@@ -278,8 +277,7 @@ Thus, this does not include the current directory.")
     (let* ((letter (match-string 1))
 	   (regexp (concat "\\`" letter))
 	   (path (eshell-find-previous-directory regexp)))
-      (concat (or path letter)
-	      (char-to-string directory-sep-char)))))
+      (concat (or path letter) "/"))))
 
 (defun eshell-complete-user-reference ()
   "If there is a user reference, complete it."
@@ -302,7 +300,7 @@ Thus, this does not include the current directory.")
   (let* ((path default-directory)
 	 (len (length path)))
     (if (and (> len 1)
-	     (eq (aref path (1- len)) directory-sep-char)
+	     (eq (aref path (1- len)) ?/)
 	     (not (and (eshell-under-windows-p)
 		       (string-match "\\`[A-Za-z]:[\\\\/]\\'" path))))
 	(setq path (substring path 0 (1- (length path)))))
@@ -321,14 +319,12 @@ in the minibuffer:
     (before translate-multiple-dots
 	    (filename &optional directory) activate)
     (setq filename (eshell-expand-multiple-dots filename)))"
-  (while (string-match "\\.\\.\\(\\.+\\)" path)
+  (while (string-match "\\(?:^\\|/\\)\\.\\.\\(\\.+\\)\\(?:$\\|/\\)" path)
     (let* ((extra-dots (match-string 1 path))
 	   (len (length extra-dots))
 	   replace-text)
       (while (> len 0)
-	(setq replace-text
-	      (concat replace-text
-		      (char-to-string directory-sep-char) "..")
+	(setq replace-text (concat replace-text "/..")
 	      len (1- len)))
       (setq path
 	    (replace-match replace-text t t path 1))))
@@ -373,7 +369,7 @@ in the minibuffer:
 	(setq path
 	      (ring-remove eshell-last-dir-ring
 			   (if index
-			       (string-to-int index)
+			       (string-to-number index)
 			     0)))))
      ((and path (string-match "^=\\(.*\\)$" path))
       (let ((oldpath (eshell-find-previous-directory
@@ -572,4 +568,5 @@ in the minibuffer:
 
 ;;; Code:
 
+;;; arch-tag: 1e9c5a95-f1bd-45f8-ad36-55aac706e787
 ;;; em-dirs.el ends here
