@@ -1,15 +1,16 @@
-;;; shadowfile.el --- automatic file copying for Emacs 19
+;;; shadowfile.el --- automatic file copying
 
-;; Copyright (C) 1993, 1994 Free Software Foundation, Inc.
+;; Copyright (C) 1993, 1994, 2001, 2002, 2003, 2004,
+;;   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 
-;; Author: Boris Goldowsky <boris@gnu.ai.mit.edu>
-;; Keywords: comm
+;; Author: Boris Goldowsky <boris@gnu.org>
+;; Keywords: comm files
 
 ;; This file is part of GNU Emacs.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
+;; the Free Software Foundation; either version 3, or (at your option)
 ;; any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
@@ -19,10 +20,10 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
-;; Commentary:
+;;; Commentary:
 
 ;;  This package helps you to keep identical copies of files in more than one
 ;;  place - possibly on different machines.  When you save a file, it checks
@@ -31,11 +32,10 @@
 
 ;; Installation & Use:
 
-;;  Put (require 'shadowfile) in your .emacs; add clusters (if necessary)
-;;  and file groups with shadow-define-cluster,
+;;  Add clusters (if necessary) and file groups with shadow-define-cluster,
 ;;  shadow-define-literal-group, and shadow-define-regexp-group (see the
-;;  documentation for these functions for information on how and when to
-;;  use them).  After doing this once, everything should be automatic.
+;;  documentation for these functions for information on how and when to use
+;;  them).  After doing this once, everything should be automatic.
 
 ;;  The lists of clusters and shadows are saved in a file called .shadows,
 ;;  so that they can be remembered from one emacs session to another, even
@@ -71,12 +71,12 @@
 ;;  - It will not make directories for you, it just fails to copy files
 ;;  that belong in non-existent directories.
 ;;
-;;  Please report any bugs to me (boris@gnu.ai.mit.edu).  Also let me know
+;;  Please report any bugs to me (boris@gnu.org).  Also let me know
 ;;  if you have suggestions or would like to be informed of updates.
 
+
 ;;; Code:
 
-(provide 'shadowfile)
 (require 'ange-ftp)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -86,6 +86,7 @@
 (defgroup shadow nil
   "Automatic file copying when saving a file."
   :prefix "shadow-"
+  :link '(emacs-commentary-link "shadowfile")
   :group 'files)
 
 (defcustom shadow-noquery nil
@@ -96,20 +97,20 @@ is no buffer currently visiting the file."
   :group 'shadow)
 
 (defcustom shadow-inhibit-message nil
-  "*If nonnil, do not display a message when a file needs copying."
+  "*If non-nil, do not display a message when a file needs copying."
   :type 'boolean
   :group 'shadow)
 
 (defcustom shadow-inhibit-overload nil
-  "If nonnil, shadowfile won't redefine C-x C-c.
-Normally it overloads the function `save-buffers-kill-emacs' to check
-for files have been changed and need to be copied to other systems."
+  "If non-nil, shadowfile won't redefine \\[save-buffers-kill-emacs].
+Normally it overloads the function `save-buffers-kill-emacs' to check for
+files that have been changed and need to be copied to other systems."
   :type 'boolean
   :group 'shadow)
 
 (defcustom shadow-info-file nil
-  "File to keep shadow information in.  
-The shadow-info-file should be shadowed to all your accounts to
+  "File to keep shadow information in.
+The `shadow-info-file' should be shadowed to all your accounts to
 ensure consistency.  Default: ~/.shadows"
   :type '(choice (const nil) file)
   :group 'shadow)
@@ -117,8 +118,8 @@ ensure consistency.  Default: ~/.shadows"
 (defcustom shadow-todo-file nil
   "File to store the list of uncopied shadows in.
 This means that if a remote system is down, or for any reason you cannot or
-decide not to copy your shadow files at the end of one emacs session, it will
-remember and ask you again in your next emacs session.
+decide not to copy your shadow files at the end of one Emacs session, it will
+remember and ask you again in your next Emacs session.
 This file must NOT be shadowed to any other system, it is host-specific.
 Default: ~/.shadow_todo"
   :type '(choice (const nil) file)
@@ -140,17 +141,17 @@ Default: ~/.shadow_todo"
 ;;;
 
 (defvar shadow-clusters nil
-  "List of host clusters \(see shadow-define-cluster).")
+  "List of host clusters \(see `shadow-define-cluster').")
 
 (defvar shadow-literal-groups nil
   "List of files that are shared between hosts.
 This list contains shadow structures with literal filenames, created by
-shadow-define-group.")
+`shadow-define-literal-group'.")
 
 (defvar shadow-regexp-groups nil
   "List of file types that are shared between hosts.
-This list contains shadow structures with regexps matching filenames, 
-created by shadow-define-regexp-group.")
+This list contains shadow structures with regexps matching filenames,
+created by `shadow-define-regexp-group'.")
 
 ;;;
 ;;; Other internal variables:
@@ -168,14 +169,8 @@ created by shadow-define-regexp-group.")
 ;;; Syntactic sugar; General list and string manipulation
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defmacro shadow-when (condition &rest body)
-  ;; From cl.el
-  "(shadow-when CONDITION . BODY) => evaluate BODY if CONDITION is true."
-  (` (if (not (, condition))  ()  (,@ body))))
-  
 (defun shadow-union (a b)
-  "Add members of list A to list B
-if they are not equal to items already in B."
+  "Add members of list A to list B if not equal to items already in B."
   (if (null a)
       b
     (if (member (car a) b)
@@ -183,8 +178,7 @@ if they are not equal to items already in B."
       (shadow-union (cdr a) (cons (car a) b)))))
 
 (defun shadow-find (func list)
-  "If FUNC applied to some element of LIST is nonnil, 
-return the first such element."
+  "If FUNC applied to some element of LIST is non-nil, return first such element."
   (while (and list (not (funcall func (car list))))
     (setq list (cdr list)))
   (car list))
@@ -205,13 +199,13 @@ Nondestructive; actually returns a copy of the list with the elements removed."
 	((concat (car strings) " " (shadow-join (cdr strings) sep)))))
 
 (defun shadow-regexp-superquote (string)
-  "Like regexp-quote, but includes the ^ and $ 
-to make sure regexp matches nothing but STRING."
+  "Like `regexp-quote', but includes the ^ and $.
+This makes sure regexp matches nothing but STRING."
   (concat "^" (regexp-quote string) "$"))
 
 (defun shadow-suffix (prefix string)
   "If PREFIX begins STRING, return the rest.
-Return value is nonnil if PREFIX and STRING are string= up to the length of
+Return value is non-nil if PREFIX and STRING are `string=' up to the length of
 PREFIX."
   (let ((lp (length prefix))
 	(ls (length string)))
@@ -228,12 +222,12 @@ PREFIX."
 ;;; either.
 
 (defun shadow-make-cluster (name primary regexp)
-  "Creates a shadow cluster 
-called NAME, using the PRIMARY hostname, REGEXP matching all hosts in the
-cluster.  The variable shadow-clusters associates the names of clusters to
-these structures. 
-   This function is for program use: to create clusters interactively, use
-shadow-define-cluster instead."
+  "Create a shadow cluster.
+It is called NAME, uses the PRIMARY hostname and REGEXP matching all
+hosts in the cluster.  The variable `shadow-clusters' associates the
+names of clusters to these structures.  This function is for program
+use: to create clusters interactively, use `shadow-define-cluster'
+instead."
   (list name primary regexp))
 
 (defmacro shadow-cluster-name (cluster)
@@ -249,14 +243,14 @@ shadow-define-cluster instead."
   (list 'elt cluster 2))
 
 (defun shadow-set-cluster (name primary regexp)
-  "Put cluster NAME on the list of clusters,
-replacing old definition if any.  PRIMARY and REGEXP are the
+  "Put cluster NAME on the list of clusters.
+Replace old definition, if any.  PRIMARY and REGEXP are the
 information defining the cluster.  For interactive use, call
-shadow-define-cluster instead."
+`shadow-define-cluster' instead."
   (let ((rest (shadow-remove-if
 	       (function (lambda (x) (equal name (car x))))
 	       shadow-clusters)))
-    (setq shadow-clusters 
+    (setq shadow-clusters
 	  (cons (shadow-make-cluster name primary regexp)
 		rest))))
 
@@ -274,8 +268,7 @@ shadow-define-cluster instead."
 ;;; SITES
 
 (defun shadow-site-cluster (site)
-  "Given a SITE \(hostname or cluster name), return the cluster
-that it is in, or nil."
+  "Given a SITE \(hostname or cluster name), return cluster it is in, or nil."
   (or (assoc site shadow-clusters)
       (shadow-find
        (function (lambda (x)
@@ -292,9 +285,9 @@ that it is in, or nil."
       ans)))
 
 (defun shadow-site-match (site1 site2)
-  "Nonnil iff SITE1 is or includes SITE2.  
-Each may be a host or cluster name; if they are clusters, regexp of site1 will
-be matched against the primary of site2."
+  "Non-nil if SITE1 is or includes SITE2.
+Each may be a host or cluster name; if they are clusters, regexp of SITE1 will
+be matched against the primary of SITE2."
   (or (string-equal site1 site2) ; quick check
       (let* ((cluster1 (shadow-get-cluster site1))
 	     (primary2 (shadow-site-primary site2)))
@@ -303,47 +296,47 @@ be matched against the primary of site2."
 	  (string-equal site1 primary2)))))
 
 (defun shadow-get-user (site)
-  "Returns the default username for a site."
+  "Return the default username for a SITE."
   (ange-ftp-get-user (shadow-site-primary site)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Filename manipulation
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun shadow-parse-fullpath (fullpath)
-  "Parse PATH into \(site user path) list,
-or leave it alone if it already is one.  Returns nil if the argument is not a
-full ange-ftp pathname."
-  (if (listp fullpath)
-      fullpath
-    (ange-ftp-ftp-name fullpath)))
+(defun shadow-parse-fullname (fullname)
+  "Parse FULLNAME into \(site user path) list.
+Leave it alone if it already is one.  Return nil if the argument is
+not a full ange-ftp pathname."
+  (if (listp fullname)
+      fullname
+    (ange-ftp-ftp-name fullname)))
 
-(defun shadow-parse-path (path)
-  "Parse any PATH into \(site user path) list.
-Argument can be a simple path, full ange-ftp path, or already a hup list."
-  (or (shadow-parse-fullpath path)
+(defun shadow-parse-name (name)
+  "Parse any NAME into \(site user name) list.
+Argument can be a simple name, full ange-ftp name, or already a hup list."
+  (or (shadow-parse-fullname name)
       (list shadow-system-name
 	    (user-login-name)
-	    path)))
+	    name)))
 
-(defsubst shadow-make-fullpath (host user path)
-  "Make an ange-ftp style fullpath out of HOST, USER (optional), and PATH.
+(defsubst shadow-make-fullname (host user name)
+  "Make an ange-ftp style fullname out of HOST, USER (optional), and NAME.
 This is probably not as general as it ought to be."
-  (concat "/" 
+  (concat "/"
 	  (if user (concat user "@"))
 	  host ":"
-	  path))
+	  name))
 
-(defun shadow-replace-path-component (fullpath newpath)
-  "Return FULLPATH with the pathname component changed to NEWPATH."
-  (let ((hup (shadow-parse-fullpath fullpath)))
-    (shadow-make-fullpath (nth 0 hup) (nth 1 hup) newpath)))
+(defun shadow-replace-name-component (fullname newname)
+  "Return FULLNAME with the name component changed to NEWNAME."
+  (let ((hup (shadow-parse-fullname fullname)))
+    (shadow-make-fullname (nth 0 hup) (nth 1 hup) newname)))
 
 (defun shadow-local-file (file)
-  "If FILENAME is at this site,
-remove /user@host part.  If refers to a different system or a different user on
-this system, return nil."
-  (let ((hup (shadow-parse-fullpath file)))
+  "If FILE is at this site, remove /user@host part.
+If refers to a different system or a different user on this system,
+return nil."
+  (let ((hup (shadow-parse-fullname file)))
     (cond ((null hup) file)
 	  ((and (shadow-site-match (nth 0 hup) shadow-system-name)
 		(string-equal (nth 1 hup) (user-login-name)))
@@ -351,91 +344,92 @@ this system, return nil."
 	  (t nil))))
 
 (defun shadow-expand-cluster-in-file-name (file)
-  "If hostname part of FILE is a cluster, expand it
-into the cluster's primary hostname.  Will return the pathname bare if it is
-a local file."
-  (let ((hup (shadow-parse-path file))
+  "If hostname part of FILE is a cluster, expand it to cluster's primary hostname.
+Will return the name bare if it is a local file."
+  (let ((hup (shadow-parse-name file))
 	cluster)
     (cond ((null hup) file)
 	  ((shadow-local-file hup))
-	  ((shadow-make-fullpath (shadow-site-primary (nth 0 hup))
+	  ((shadow-make-fullname (shadow-site-primary (nth 0 hup))
 				 (nth 1 hup)
 				 (nth 2 hup))))))
 
 (defun shadow-expand-file-name (file &optional default)
-  "Expand file name and get file's true name."
+  "Expand file name and get FILE's true name."
   (file-truename (expand-file-name file default)))
 
 (defun shadow-contract-file-name (file)
-  "Simplify FILENAME
-by replacing (when possible) home directory with ~, and hostname with cluster
-name that includes it.  Filename should be absolute and true."
-  (let* ((hup (shadow-parse-path file))
+  "Simplify FILE.
+Do so by replacing (when possible) home directory with ~, and hostname
+with cluster name that includes it.  Filename should be absolute and
+true."
+  (let* ((hup (shadow-parse-name file))
 	 (homedir (if (shadow-local-file hup)
 		      shadow-homedir
 		    (file-name-as-directory
-		     (nth 2 (shadow-parse-fullpath 
+		     (nth 2 (shadow-parse-fullname
 			     (expand-file-name
-			      (shadow-make-fullpath
+			      (shadow-make-fullname
 			       (nth 0 hup) (nth 1 hup) "~")))))))
 	 (suffix (shadow-suffix homedir (nth 2 hup)))
 	 (cluster (shadow-site-cluster (nth 0 hup))))
-    (shadow-make-fullpath
+    (shadow-make-fullname
      (if cluster
 	 (shadow-cluster-name cluster)
        (nth 0 hup))
      (nth 1 hup)
-     (if suffix 
+     (if suffix
 	 (concat "~/" suffix)
        (nth 2 hup)))))
 
 (defun shadow-same-site (pattern file)
   "True if the site of PATTERN and of FILE are on the same site.
 If usernames are supplied, they must also match exactly.  PATTERN and FILE may
-be lists of host, user, path, or ange-ftp pathnames.  FILE may also be just a
+be lists of host, user, name, or ange-ftp file names.  FILE may also be just a
 local filename."
-  (let ((pattern-sup (shadow-parse-fullpath pattern))
-	(file-sup    (shadow-parse-path file)))
+  (let ((pattern-sup (shadow-parse-fullname pattern))
+	(file-sup    (shadow-parse-name file)))
     (and
      (shadow-site-match (nth 0 pattern-sup) (nth 0 file-sup))
      (or (null (nth 1 pattern-sup))
 	 (string-equal (nth 1 pattern-sup) (nth 1 file-sup))))))
 
 (defun shadow-file-match (pattern file &optional regexp)
- "Returns t if PATTERN matches FILE.
-If REGEXP is supplied and nonnil, the pathname part of the pattern is a regular
+ "Return t if PATTERN matches FILE.
+If REGEXP is supplied and non-nil, the file part of the pattern is a regular
 expression, otherwise it must match exactly.  The sites and usernames must
-match---see shadow-same-site.  The pattern must be in full ange-ftp format, but
-the file can be any valid filename.  This function does not do any filename
-expansion or contraction, you must do that yourself first."
- (let* ((pattern-sup (shadow-parse-fullpath pattern))
-	(file-sup (shadow-parse-path file)))
+match---see `shadow-same-site'.  The pattern must be in full ange-ftp format,
+but the file can be any valid filename.  This function does not do any
+filename expansion or contraction, you must do that yourself first."
+ (let* ((pattern-sup (shadow-parse-fullname pattern))
+	(file-sup (shadow-parse-name file)))
    (and (shadow-same-site pattern-sup file-sup)
-	(if regexp 
+	(if regexp
 	    (string-match (nth 2 pattern-sup) (nth 2 file-sup))
 	  (string-equal (nth 2 pattern-sup) (nth 2 file-sup))))))
- 
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; User-level Commands
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+;;;###autoload
 (defun shadow-define-cluster (name)
-  "Edit \(or create) the definition of a cluster.
+  "Edit \(or create) the definition of a cluster NAME.
 This is a group of hosts that share directories, so that copying to or from
 one of them is sufficient to update the file on all of them.  Clusters are
 defined by a name, the network address of a primary host \(the one we copy
-files to), and a regular expression that matches the hostnames of all the sites
-in the cluster."
+files to), and a regular expression that matches the hostnames of all the
+sites in the cluster."
   (interactive (list (completing-read "Cluster name: " shadow-clusters () ())))
   (let* ((old (shadow-get-cluster name))
 	 (primary (read-string "Primary host: "
-			       (if old (shadow-cluster-primary old) 
+			       (if old (shadow-cluster-primary old)
 				 name)))
 	 (regexp   (let (try-regexp)
 		     (while (not
-			     (string-match 
+			     (string-match
 			      (setq try-regexp
-				    (read-string 
+				    (read-string
 				     "Regexp matching all host names: "
 				     (if old (shadow-cluster-regexp old)
 				       (shadow-regexp-superquote primary))))
@@ -443,8 +437,8 @@ in the cluster."
 		       (message "Regexp doesn't include the primary host!")
 		       (sit-for 2))
 		     try-regexp))
-;	 (username (read-no-blanks-input 
-;		    (format "Username [default: %s]: " 
+;	 (username (read-no-blanks-input
+;		    (format "Username (default %s): "
 ;			    (shadow-get-user primary))
 ;		    (if old (or (shadow-cluster-username old) "")
 ;		      (user-login-name))))
@@ -452,57 +446,59 @@ in the cluster."
 ;    (if (string-equal "" username) (setq username nil))
     (shadow-set-cluster name primary regexp)))
 
+;;;###autoload
 (defun shadow-define-literal-group ()
   "Declare a single file to be shared between sites.
 It may have different filenames on each site.  When this file is edited, the
 new version will be copied to each of the other locations.  Sites can be
-specific hostnames, or names of clusters \(see shadow-define-cluster)."
+specific hostnames, or names of clusters \(see `shadow-define-cluster')."
   (interactive)
-  (let* ((hup (shadow-parse-fullpath 
+  (let* ((hup (shadow-parse-fullname
 	       (shadow-contract-file-name (buffer-file-name))))
-	 (path (nth 2 hup))
+	 (name (nth 2 hup))
 	 user site group)
     (while (setq site (shadow-read-site))
-      (setq user (read-string (format "Username [default %s]: "
+      (setq user (read-string (format "Username (default %s): "
 				      (shadow-get-user site)))
-	    path (read-string "Filename: " path))
-      (setq group (cons (shadow-make-fullpath site 
+	    name (read-string "Filename: " name))
+      (setq group (cons (shadow-make-fullname site
 					      (if (string-equal "" user)
 						  (shadow-get-user site)
 						user)
-					      path)
+					      name)
 			group)))
     (setq shadow-literal-groups (cons group shadow-literal-groups)))
   (shadow-write-info-file))
 
+;;;###autoload
 (defun shadow-define-regexp-group ()
   "Make each of a group of files be shared between hosts.
 Prompts for regular expression; files matching this are shared between a list
-of sites, which are also prompted for. The filenames must be identical on all
-hosts \(if they aren't, use shadow-define-group instead of this function).
-Each site can be either a hostname or the name of a cluster \(see
-shadow-define-cluster)."
+of sites, which are also prompted for.  The filenames must be identical on all
+hosts \(if they aren't, use `shadow-define-literal-group' instead of this
+function).  Each site can be either a hostname or the name of a cluster \(see
+`shadow-define-cluster')."
   (interactive)
-  (let ((regexp (read-string 
-		 "Filename regexp: " 
+  (let ((regexp (read-string
+		 "Filename regexp: "
 		 (if (buffer-file-name)
 		     (shadow-regexp-superquote
 		      (nth 2
-			   (shadow-parse-path
+			   (shadow-parse-name
 			    (shadow-contract-file-name
 			     (buffer-file-name))))))))
 	site sites usernames)
     (while (setq site (shadow-read-site))
       (setq sites (cons site sites))
-      (setq usernames 
+      (setq usernames
 	    (cons (read-string (format "Username for %s: " site)
 			       (shadow-get-user site))
 		  usernames)))
-    (setq shadow-regexp-groups 
+    (setq shadow-regexp-groups
 	  (cons (shadow-make-group regexp sites usernames)
 		shadow-regexp-groups))
     (shadow-write-info-file)))
-    
+
 (defun shadow-shadows ()
   ;; Mostly for debugging.
   "Interactive function to display shadows of a buffer."
@@ -511,20 +507,21 @@ shadow-define-cluster)."
 				  (shadow-shadows-of (buffer-file-name)))
 			  " ")))
     (message "%s"
-	     (if (zerop (length msg)) 
+	     (if (zerop (length msg))
 		 "No shadows."
 	       msg))))
 
 (defun shadow-copy-files (&optional arg)
   "Copy all pending shadow files.
 With prefix argument, copy all pending files without query.
-Pending copies are stored in variable shadow-files-to-copy, and in
-shadow-todo-file if necessary.  This function is invoked by
-shadow-save-buffers-kill-emacs, so it is not usually necessary to
+Pending copies are stored in variable `shadow-files-to-copy', and in
+`shadow-todo-file' if necessary.  This function is invoked by
+`shadow-save-buffers-kill-emacs', so it is not usually necessary to
 call it manually."
   (interactive "P")
-  (if (and (not shadow-files-to-copy) (interactive-p))
-      (message "No files need to be shadowed.")
+  (if (not shadow-files-to-copy)
+      (if (interactive-p)
+	  (message "No files need to be shadowed."))
     (save-excursion
       (map-y-or-n-p (function
 		     (lambda (pair)
@@ -539,17 +536,17 @@ call it manually."
   "Cancel the instruction to copy some files.
 Prompts for which copy operations to cancel.  You will not be asked to copy
 them again, unless you make more changes to the files.  To cancel a shadow
-permanently, remove the group from shadow-literal-groups or
-shadow-regexp-groups."
+permanently, remove the group from `shadow-literal-groups' or
+`shadow-regexp-groups'."
   (interactive)
   (map-y-or-n-p (function (lambda (pair)
-			    (format "Cancel copying %s to %s? " 
+			    (format "Cancel copying %s to %s? "
 				    (car pair) (cdr pair))))
-		(function (lambda (pair) 
+		(function (lambda (pair)
 			    (shadow-remove-from-todo pair)))
 		shadow-files-to-copy
 		'("shadow" "shadows" "cancel copy"))
-  (message "There are %d shadows to be updated." 
+  (message "There are %d shadows to be updated."
 	   (length shadow-files-to-copy))
   (shadow-write-todo-file))
 
@@ -558,45 +555,45 @@ shadow-regexp-groups."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defun shadow-make-group (regexp sites usernames)
-  "Makes a description of a file group---
+  "Make a description of a file group---
 actually a list of regexp ange-ftp file names---from REGEXP \(name of file to
 be shadowed), list of SITES, and corresponding list of USERNAMES for each
 site."
   (if sites
-      (cons (shadow-make-fullpath (car sites) (car usernames) regexp)
+      (cons (shadow-make-fullname (car sites) (car usernames) regexp)
 	    (shadow-make-group regexp (cdr sites) (cdr usernames)))
     nil))
 
 (defun shadow-copy-file (s)
   "Copy one shadow file."
-  (let* ((buffer 
-	  (cond ((get-file-buffer 
+  (let* ((buffer
+	  (cond ((get-file-buffer
 		  (abbreviate-file-name (shadow-expand-file-name (car s)))))
 		((not (file-readable-p (car s)))
 		 (if (y-or-n-p
-		      (format "Cannot find file %s--cancel copy request?"
+		      (format "Cannot find file %s--cancel copy request? "
 			      (car s)))
 		     (shadow-remove-from-todo s))
 		 nil)
 		((or (eq t shadow-noquery)
-		     (y-or-n-p 
-		      (format "No buffer for %s -- update shadow anyway?"
+		     (y-or-n-p
+		      (format "No buffer for %s -- update shadow anyway? "
 			      (car s))))
 		 (find-file-noselect (car s)))))
 	 (to (shadow-expand-cluster-in-file-name (cdr s))))
-    (shadow-when buffer
+    (when buffer
       (set-buffer buffer)
       (save-restriction
 	(widen)
-	(condition-case i 
+	(condition-case i
 	    (progn
 	      (write-region (point-min) (point-max) to)
 	      (shadow-remove-from-todo s))
 	  (error (message "Shadow %s not updated!" (cdr s))))))))
 
 (defun shadow-shadows-of (file)
-  "Returns copy operations needed to update FILE.
-Filename should have clusters expanded, but otherwise can have any format.  
+  "Return copy operations needed to update FILE.
+Filename should have clusters expanded, but otherwise can have any format.
 Return value is a list of dotted pairs like \(from . to), where from
 and to are absolute file names."
   (or (symbol-value (intern-soft file shadow-hashtable))
@@ -604,7 +601,7 @@ and to are absolute file names."
 			     (or (shadow-local-file file) file)
 			     shadow-homedir))
 	     (canonical-file (shadow-contract-file-name absolute-file))
-	     (shadows 
+	     (shadows
 	      (mapcar (function (lambda (shadow)
 				  (cons absolute-file shadow)))
 		      (append
@@ -615,34 +612,33 @@ and to are absolute file names."
 	(set (intern file shadow-hashtable) shadows))))
 
 (defun shadow-shadows-of-1 (file groups regexp)
-  "Return list of FILE's shadows in GROUPS, 
-which are considered as regular expressions if third arg REGEXP is true."
+  "Return list of FILE's shadows in GROUPS.
+Consider them as regular expressions if third arg REGEXP is true."
   (if groups
       (let ((nonmatching
-	     (shadow-remove-if 
+	     (shadow-remove-if
 	      (function (lambda (x) (shadow-file-match x file regexp)))
 	      (car groups))))
 	(append (cond ((equal nonmatching (car groups)) nil)
-		      (regexp 
-		       (let ((realpath (nth 2 (shadow-parse-fullpath file))))
-			 (mapcar 
-			  (function 
-			   (lambda (x) 
-			     (shadow-replace-path-component x realpath)))
+		      (regexp
+		       (let ((realname (nth 2 (shadow-parse-fullname file))))
+			 (mapcar
+			  (function
+			   (lambda (x)
+			     (shadow-replace-name-component x realname)))
 			  nonmatching)))
 		      (t nonmatching))
 		(shadow-shadows-of-1 file (cdr groups) regexp)))))
 
 (defun shadow-add-to-todo ()
-  "If current buffer has shadows, add them to the list
-of files needing to be copied."
-  (let ((shadows (shadow-shadows-of 
-		  (shadow-expand-file-name 
+  "If current buffer has shadows, add them to the list needing to be copied."
+  (let ((shadows (shadow-shadows-of
+		  (shadow-expand-file-name
 		   (buffer-file-name (current-buffer))))))
-    (shadow-when shadows
+    (when shadows
       (setq shadow-files-to-copy
 	    (shadow-union shadows shadow-files-to-copy))
-      (shadow-when (not shadow-inhibit-message)
+      (when (not shadow-inhibit-message)
 	(message "%s" (substitute-command-keys
 		       "Use \\[shadow-copy-files] to update shadows."))
 	(sit-for 1))
@@ -650,55 +646,55 @@ of files needing to be copied."
   nil)     ; Return nil for write-file-hooks
 
 (defun shadow-remove-from-todo (pair)
-  "Remove PAIR from shadow-files-to-copy.
-PAIR must be (eq to) one of the elements of that list."
-  (setq shadow-files-to-copy 
+  "Remove PAIR from `shadow-files-to-copy'.
+PAIR must be `eq' to one of the elements of that list."
+  (setq shadow-files-to-copy
 	(shadow-remove-if (function (lambda (s) (eq s pair)))
 			  shadow-files-to-copy)))
 
 (defun shadow-read-files ()
-  "Visits and loads shadow-info-file and shadow-todo-file,
-thus restoring shadowfile's state from your last emacs session.
-Returns t unless files were locked; then returns nil."
+  "Visit and load `shadow-info-file' and `shadow-todo-file'.
+Thus restores shadowfile's state from your last Emacs session.
+Return t unless files were locked; then return nil."
   (interactive)
   (if (and (fboundp 'file-locked-p)
 	   (or (stringp (file-locked-p shadow-info-file))
 	       (stringp (file-locked-p shadow-todo-file))))
       (progn
-	(message "Shadowfile is running in another emacs; can't have two.")
+	(message "Shadowfile is running in another Emacs; can't have two.")
 	(beep)
 	(sit-for 3)
 	nil)
     (save-excursion
-      (shadow-when shadow-info-file
+      (when shadow-info-file
 	(set-buffer (setq shadow-info-buffer
 			  (find-file-noselect shadow-info-file)))
-	(shadow-when (and (not (buffer-modified-p))
-			  (file-newer-than-file-p (make-auto-save-file-name)
-						  shadow-info-file))
+	(when (and (not (buffer-modified-p))
+		   (file-newer-than-file-p (make-auto-save-file-name)
+					   shadow-info-file))
 	  (erase-buffer)
-	  (message "Data recovered from %s." 
+	  (message "Data recovered from %s."
 		   (car (insert-file-contents (make-auto-save-file-name))))
 	  (sit-for 1))
-	(eval-current-buffer))
-      (shadow-when shadow-todo-file
-	(set-buffer (setq shadow-todo-buffer 
+	(eval-buffer))
+      (when shadow-todo-file
+	(set-buffer (setq shadow-todo-buffer
 			  (find-file-noselect shadow-todo-file)))
-	(shadow-when (and (not (buffer-modified-p))
-			  (file-newer-than-file-p (make-auto-save-file-name)
-						  shadow-todo-file))
+	(when (and (not (buffer-modified-p))
+		   (file-newer-than-file-p (make-auto-save-file-name)
+					   shadow-todo-file))
 	  (erase-buffer)
-	  (message "Data recovered from %s." 
+	  (message "Data recovered from %s."
 		   (car (insert-file-contents (make-auto-save-file-name))))
 	  (sit-for 1))
-	(eval-current-buffer nil))
+	(eval-buffer nil))
       (shadow-invalidate-hashtable))
     t))
 
 (defun shadow-write-info-file ()
-  "Write out information to shadow-info-file.
-Also clears shadow-hashtable, since when there are new shadows defined, the old
-hashtable info is invalid."
+  "Write out information to `shadow-info-file'.
+Also clear `shadow-hashtable', since when there are new shadows
+defined, the old hashtable info is invalid."
   (shadow-invalidate-hashtable)
   (if shadow-info-file
       (save-excursion
@@ -711,8 +707,8 @@ hashtable info is invalid."
 	(shadow-insert-var 'shadow-regexp-groups))))
 
 (defun shadow-write-todo-file (&optional save)
-  "Write out information to shadow-todo-file.  
-With nonnil argument also saves the buffer."
+  "Write out information to `shadow-todo-file'.
+With non-nil argument also saves the buffer."
   (save-excursion
     (if (not shadow-todo-buffer)
 	(setq shadow-todo-buffer (find-file-noselect shadow-todo-file)))
@@ -725,7 +721,7 @@ With nonnil argument also saves the buffer."
   (if (and shadow-todo-buffer (buffer-modified-p shadow-todo-buffer))
       (save-excursion
 	(set-buffer shadow-todo-buffer)
-	(condition-case nil		; have to continue even in case of 
+	(condition-case nil		; have to continue even in case of
 	    (basic-save-buffer)		; error, otherwise kill-emacs might
 	  (error			; not work!
 	   (message "WARNING: Can't save shadow todo file; it is locked!")
@@ -735,13 +731,14 @@ With nonnil argument also saves the buffer."
   (setq shadow-hashtable (make-vector 37 0)))
 
 (defun shadow-insert-var (variable)
-  "Prettily insert a setq command for VARIABLE.
-which, when later evaluated, will restore it to its current setting.
-SYMBOL must be the name of a variable whose value is a list."
+  "Build a `setq' to restore VARIABLE.
+Prettily insert a `setq' command which, when later evaluated,
+will restore VARIABLE to its current setting.
+VARIABLE must be the name of a variable whose value is a list."
   (let ((standard-output (current-buffer)))
     (insert (format "(setq %s" variable))
     (cond ((consp (eval variable))
-	   (insert "\n  '(") 
+	   (insert "\n  '(")
 	   (prin1 (car (eval variable)))
 	   (let ((rest (cdr (eval variable))))
 	     (while rest
@@ -781,10 +778,8 @@ look for files that have been changed and need to be copied to other systems."
 	   (let ((processes (process-list))
 		 active)
 	     (while processes
-	       (and (memq (process-status (car processes)) '(run stop open))
-		    (let ((val (process-kill-without-query (car processes))))
-		      (process-kill-without-query (car processes) val)
-		      val)
+	       (and (memq (process-status (car processes)) '(run stop open listen))
+		    (process-query-on-exit-flag (car processes))
 		    (setq active t))
 	       (setq processes (cdr processes)))
 	     (or (not active)
@@ -792,61 +787,62 @@ look for files that have been changed and need to be copied to other systems."
        (kill-emacs)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Lucid Emacs compatibility 
+;;; Lucid Emacs compatibility
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;; This is on hold until someone tells me about a working version of
 ;; map-ynp for Lucid Emacs.
 
-;(shadow-when (string-match "Lucid" emacs-version)
+;(when (string-match "Lucid" emacs-version)
 ;  (require 'symlink-fix)
 ;  (require 'ange-ftp)
 ;  (require 'map-ynp)
 ;  (if (not (fboundp 'file-truename))
-;      (fset 'shadow-expand-file-name 
+;      (fset 'shadow-expand-file-name
 ;	    (symbol-function 'symlink-expand-file-name)))
 ;  (if (not (fboundp 'ange-ftp-ftp-name))
 ;      (fset 'ange-ftp-ftp-name
-;	    (symbol-function 'ange-ftp-ftp-path))))
+;	    (symbol-function 'ange-ftp-ftp-name))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Hook us up
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-;;; File shadowing is activated at load time, unless this this file is
-;;; being preloaded, in which case it is added to after-init-hook.
-;;; Thanks to Richard Caley for this scheme.
-
+;;;###autoload
 (defun shadow-initialize ()
+  "Set up file shadowing."
+  (interactive)
   (if (null shadow-homedir)
       (setq shadow-homedir
 	    (file-name-as-directory (shadow-expand-file-name "~"))))
   (if (null shadow-info-file)
-      (setq shadow-info-file 
+      (setq shadow-info-file
 	    (shadow-expand-file-name "~/.shadows")))
   (if (null shadow-todo-file)
-      (setq shadow-todo-file 
+      (setq shadow-todo-file
 	    (shadow-expand-file-name "~/.shadow_todo")))
   (if (not (shadow-read-files))
       (progn
 	(message "Shadowfile information files not found - aborting")
 	(beep)
 	(sit-for 3))
-    (shadow-when (and (not shadow-inhibit-overload)
-		      (not (fboundp 'shadow-orig-save-buffers-kill-emacs)))
-      (fset 'shadow-orig-save-buffers-kill-emacs 
-	    (symbol-function 'save-buffers-kill-emacs))
-      (fset 'save-buffers-kill-emacs
-	    (symbol-function 'shadow-save-buffers-kill-emacs)))
+    (when (and (not shadow-inhibit-overload)
+	       (not (fboundp 'shadow-orig-save-buffers-kill-emacs)))
+      (defalias 'shadow-orig-save-buffers-kill-emacs
+	(symbol-function 'save-buffers-kill-emacs))
+      (defalias 'save-buffers-kill-emacs 'shadow-save-buffers-kill-emacs))
     (add-hook 'write-file-hooks 'shadow-add-to-todo)
     (define-key ctl-x-4-map "s" 'shadow-copy-files)))
 
-(if noninteractive
-    (add-hook 'after-init-hook 'shadow-initialize)
-  (shadow-initialize))
+(defun shadowfile-unload-hook ()
+  (if (fboundp 'shadow-orig-save-buffers-kill-emacs)
+      (fset 'save-buffers-kill-emacs
+	    (symbol-function 'shadow-orig-save-buffers-kill-emacs)))
+  (remove-hook 'write-file-hooks 'shadow-add-to-todo))
 
-;;;Local Variables:
-;;;eval:(put 'shadow-when 'lisp-indent-hook 1)
-;;;End:
+(add-hook 'shadowfile-unload-hook 'shadowfile-unload-hook)
 
+(provide 'shadowfile)
+
+;;; arch-tag: e2f4cdd7-2bab-4def-9130-9e69b412b79e
 ;;; shadowfile.el ends here
