@@ -1,10 +1,11 @@
-/* Copyright (C) 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1995, 2001, 2002, 2003, 2004, 2005,
+      2006, 2007, 2008  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -14,8 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.  */
 
 
 /* Workable version of <sys/socket.h> based on winsock.h */
@@ -81,8 +82,15 @@ Boston, MA 02111-1307, USA.  */
 #define inet_addr      sys_inet_addr
 #define gethostname    sys_gethostname
 #define gethostbyname  sys_gethostbyname
+#define getpeername    sys_getpeername
 #define getservbyname  sys_getservbyname
 #define shutdown       sys_shutdown
+#define setsockopt     sys_setsockopt
+#define listen         sys_listen
+#define getsockname    sys_getsockname
+#define accept         sys_accept
+#define recvfrom       sys_recvfrom
+#define sendto         sys_sendto
 
 int sys_socket(int af, int type, int protocol);
 int sys_bind (int s, const struct sockaddr *addr, int namelen);
@@ -91,9 +99,24 @@ u_short sys_htons (u_short hostshort);
 u_short sys_ntohs (u_short netshort);
 unsigned long sys_inet_addr (const char * cp);
 int sys_gethostname (char * name, int namelen);
-struct hostent * sys_gethostbyname(const char * name);
-struct servent * sys_getservbyname(const char * name, const char * proto);
+struct hostent * sys_gethostbyname (const char * name);
+struct servent * sys_getservbyname (const char * name, const char * proto);
+int sys_getpeername (int s, struct sockaddr *addr, int * namelen);
 int sys_shutdown (int socket, int how);
+int sys_setsockopt (int s, int level, int oname, const void * oval, int olen);
+int sys_listen (int s, int backlog);
+int sys_getsockname (int s, struct sockaddr * name, int * namelen);
+int sys_accept (int s, struct sockaddr *addr, int *addrlen);
+int sys_recvfrom (int s, char *buf, int len, int flags,
+		  struct sockaddr *from, int * fromlen);
+int sys_sendto (int s, const char * buf, int len, int flags,
+		const struct sockaddr *to, int tolen);
+
+/* In addition to wrappers for the winsock functions, we also provide
+   an fcntl function, for setting sockets to non-blocking mode.  */
+int fcntl (int s, int cmd, int options);
+#define F_SETFL   4
+#define O_NDELAY  04000
 
 /* we are providing a real h_errno variable */
 #undef h_errno
@@ -141,3 +164,6 @@ extern int h_errno;
 #endif /* _SOCKET_H_ */
 
 /* end of socket.h */
+
+/* arch-tag: e3b8b91c-aaa0-4bc4-be57-a85a1dd247b4
+   (do not change this comment) */
