@@ -1,5 +1,5 @@
-/* Definitions file for GNU Emacs running on bsd 4.1.
-   Copyright (C) 1985, 1986, 1999, 2001, 2002, 2003, 2004,
+/* Definitions file for GNU Emacs running on bsd 4.2
+   Copyright (C) 1985, 1986, 1994, 2001, 2002, 2003, 2004,
                  2005, 2006, 2007, 2008  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
@@ -25,9 +25,13 @@ Boston, MA 02110-1301, USA.  */
  *	Define all the symbols that apply correctly.
  */
 
-#define BSD4_1
+#ifndef BSD4_2
+#define BSD4_2 1
+#endif /* BSD4_2 */
 
-#define BSD_SYSTEM
+#ifndef BSD_SYSTEM
+#define BSD_SYSTEM 42
+#endif /* BSD_SYSTEM */
 
 /* SYSTEM_TYPE should indicate the kind of system you are using.
  It sets the Lisp variable system-type.  */
@@ -52,7 +56,7 @@ Boston, MA 02110-1301, USA.  */
  *	Define HAVE_PTYS if the system supports pty devices.
  */
 
-/* #define HAVE_PTYS */
+#define HAVE_PTYS
 
 /* Define this macro if system defines a type `union wait'.  */
 
@@ -60,18 +64,18 @@ Boston, MA 02110-1301, USA.  */
 
 /* Define HAVE_SOCKETS if system supports 4.2-compatible sockets.  */
 
-/* #define HAVE_SOCKETS */
+#define HAVE_SOCKETS
 
 /*
  *	Define NONSYSTEM_DIR_LIBRARY to make Emacs emulate
  *      The 4.2 opendir, etc., library functions.
  */
 
-#define NONSYSTEM_DIR_LIBRARY
+/* #define NONSYSTEM_DIR_LIBRARY */
 
 /* Define this symbol if your system has the functions bcopy, etc. */
 
-/* #define BSTRING */
+#define BSTRING
 
 /* subprocesses should be defined if you want to
    have code for asynchronous subprocesses
@@ -97,7 +101,7 @@ Boston, MA 02110-1301, USA.  */
    so that Emacs can tell instantly when you try to modify
    a file that someone else has modified in his Emacs.  */
 
-#undef CLASH_DETECTION    /* Might work; not tried yet.  */
+#define CLASH_DETECTION
 
 /* The file containing the kernel's symbol table is called /vmunix.  */
 
@@ -107,27 +111,14 @@ Boston, MA 02110-1301, USA.  */
    is named _avenrun.  */
 
 #define LDAV_SYMBOL "_avenrun"
-
-/* Special hacks needed to make Emacs run on this system.  */
 
-/* Make the function `signal' act as in 4.2.  */
+/* Send signals to subprocesses by "typing" special chars at them.  */
 
-#define signal sigset
+#define SIGNALS_VIA_CHARACTERS
 
-#define _longjmp longjmp
-#define _setjmp setjmp
+/* Process groups work in the traditional BSD manner.  */
 
-#define lstat stat
+#define BSD_PGRPS
 
-/* Names of flags for open.  */
-#define O_RDONLY 0
-#define O_WRONLY 1
-#define O_RDWR 2
-#define O_EXCL	2000
-#define O_CREAT 1000
-
-/* Special library needed for linking for 4.1.  */
-#define LIBS_SYSTEM -ljobs
-
-/* arch-tag: de56716a-ed9b-4851-a5c5-c882efaf9931
+/* arch-tag: 38ca640f-549e-4726-a257-71048ab4c480
    (do not change this comment) */
